#ifndef CAM_PREPROCESS_CLASS_H
#define CAM_PREPROCESS_CLASS_H

#include <cstdint>
#include <iostream>
#include <fstream>
#include <vector>
#include "stdio.h"
#include "acl.h"
#include "hi_dvpp_venc.h"
#include "hi_dvpp_vpc.h"
#include "hi_mpi_sys.h"
#include "camera.h"
#include <ara/log/logging.h>

#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/ini_parser.hpp>

class CamPreprocess
{
public:
    CamPreprocess(uint8_t id = 0);
    ~CamPreprocess();

    void init(boost::property_tree::ptree &config, uint8_t startCameraId);
    int32_t runEncoderPipeline(std::shared_ptr<hi_venc_stream> *out_enc);
    int32_t runPerceptionPipeline();
    uint32_t allocateMemory();
    void **getInputAddress();

    hi_vpc_pic_info inputPicInfo, outputPicInfo;

private:
    uint8_t id;
    bool undistortion;
    hi_vpc_crop_resize_region cropResizeRegion;
    hi_map_param mappingParameters;
    hi_remap_lut remappingTableInformation;
    hi_warp_transform_param warpTransformParam;
    hi_vpc_workspace_param workSpaceParam[1];

    unsigned int inputImageWidth, inputImageHeight, cropWidth, cropHeight, cropTopOffset, cropLeftOffset, encoderGopNumber;
    std::string remapFilePathX, remapFilePathY;

    aclrtRunMode runMode;
    uint32_t frame_ref = 0;
    int32_t epollFd;
    hi_dvpp_epoll_event event;
    int32_t fd;
    mdc::camera::CameraData cam_data;

    hi_vpc_chn channelIDs;
    hi_vpc_chn_attr channelAttr;

    aclrtContext g_context;

    void setupPipelines(boost::property_tree::ptree &config, uint8_t startCameraId);
    uint8_t initACL();
    uint8_t initMPI();

    int32_t createVPCChannel();
    int32_t createEncoderChannel(int width, int height, int bufferSize);

    void setDefaultInputPicInfo();
    void setDefaultOutputPicInfo();
    void setDefaultCropReizeRegion();
    void setDefaultRemapParam();

    uint8_t allocateByPicInfo(hi_vpc_pic_info &picInfo);
    uint8_t allocateByCropResizeRegion(hi_vpc_crop_resize_region &cropResizeRegion);
    uint8_t allocateByUndistortion(hi_warp_transform_param &warpTransformParam, hi_remap_lut &remappingTableInformation);

    int32_t getResizedCroppedImage();
    int32_t getConvertedImage();
    int32_t getUndistortedImage();
    uint8_t h265Encoder(std::shared_ptr<hi_venc_stream> *out_enc);

protected:
    ara::log::Logger &logger{ara::log::CreateLogger("PRE", "SoftwareMotion Camera PreProcess Module", ara::log::LogLevel::kInfo)};
};

#endif /* CAM_PREPROCESS_CLASS_H */