#ifndef CAMERA_HARDWARE_CLASS_H
#define CAMERA_HARDWARE_CLASS_H

#include "camera.h"
#include <vector>
#include <array>
#include <ara/log/logging.h>
#include <thread>
#include "camera_pool.h"

class CameraHardware
{
public:
    CameraHardware(std::shared_ptr<CameraPool> pool, uint8_t camStartId);
    void init(std::vector<mdc::camera::CameraAccessConfig> &configs);
    uint8_t getCameraImagesArrayVector(std::array<CameraImageResult, 4> &result);
    uint8_t getCameraYUVBuffer(void **buffer, size_t &length, mdc::camera::CameraData &image);
    void runCameras();

protected:
    ara::log::Logger &logger{ara::log::CreateLogger("SWM_CAMERA", "SoftwareMotion Camera Module", ara::log::LogLevel::kDebug)};
    bool ready = false;

private:
    int image_timeout = 1000;
    const uint8_t camStartId;
    uint8_t getRawId(uint8_t id);
    std::shared_ptr<CameraPool> pool_;
    std::thread act{};
};
#endif
