#include "camera_hardware.h"

// using namespace mdc::camera;
// using namespace mdc;

CameraHardware::CameraHardware(std::shared_ptr<CameraPool> pool, uint8_t camStartId) : pool_(pool), camStartId(camStartId)
{
}

void CameraHardware::init(std::vector<mdc::camera::CameraAccessConfig> &configs)
{
    mdc::MDCMbufSysInit();
    if (mdc::camera::CameraInit(configs))
    {
        this->logger.LogFatal() << "Camera init failed";
    }
    else
    {
        this->logger.LogDebug() << "Camera init completed";
        this->ready = true;
        this->act = std::thread(&CameraHardware::runCameras, this);
        this->logger.LogDebug() << "Camera read data thread created";
    }
}

uint8_t CameraHardware::getRawId(uint8_t id)
{
    return id % this->camStartId;
}

uint8_t CameraHardware::getCameraImagesArrayVector(std::array<CameraImageResult, 4> &result)
{
    std::vector<uint32_t> camIDs;
    this->logger.LogDebug() << "Attempt to getting camera IDs via CameraWaitEvents method. camStartId: " << this->camStartId;
    camIDs = mdc::camera::CameraWaitEvents(-1);
    if (!camIDs.empty())
    {
        for (auto id : camIDs)
        {
            uint8_t rawId = this->getRawId(id);
            std::lock_guard<std::mutex> lock(this->pool_->mtx[rawId]);
            uint8_t failed = CameraGetData(id, result[rawId].imageVector, image_timeout);
            if (!failed)
            {
                result[rawId].id = id;
                this->pool_->dataReady[rawId] = true;
                this->pool_->cv[rawId].notify_one();
                this->logger.LogDebug() << "Camera " << id << " Image successfully pushed to the vector";
            }
            else
            {
                this->logger.LogError() << "Camera" << id << "failed to get data with error " << failed;
            }
        }
    }
    else
    {
        this->logger.LogError() << "No Camera Ready";
    }
    return camIDs.size();
}

uint8_t CameraHardware::getCameraYUVBuffer(void **buffer, size_t &length, mdc::camera::CameraData &image)
{
    uint8_t retValue = image.yuvData.GetBuffData(*buffer, length);
    retValue = retValue == 0 ? 1 : 0;
    return retValue;
}

void CameraHardware::runCameras()
{
    this->logger.LogDebug() << "Thread start running";
    uint8_t cameraSize;
    while (this->ready)
    {
        if (this->pool_ != nullptr)
        {
            cameraSize = this->getCameraImagesArrayVector(this->pool_->pool);
            this->logger.LogDebug() << "number of ready camera: " << cameraSize;
        }
        else
        {
            this->logger.LogError() << "No Camera Ready";
        }
    }
}