/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2021. All rights reserved.
 * Description: MDC VIZ库 读取data文件
 * Create: 2020-09-10
 */

#ifndef VIZ_JSON_PARSER_H
#define VIZ_JSON_PARSER_H

#include <publisher.h>
#include <json/json.h>

class JsonParser {
public:
    enum ReadJsonFileReturnCode {
        OK,
        ERROR
    };
    JsonParser() = default;
    ~JsonParser() = default;


public:
    /*
     * 功能：解析Json文件中属于bool类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点bool类型的数据
     * 返回值：无
     */
    static void ParseJson(bool &elem, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于String类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点String类型的数据
     * 返回值：无
     */
    static void ParseJson(ara::core::String &elem, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于int32_t类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点int32_t类型的数据
     * 返回值：无
     */
    static void ParseJson(int32_t &elem, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于uint32_t类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点uint32_t类型的数据
     * 返回值：无
     */
    static void ParseJson(uint32_t &elem, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于uint8_t类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点uint8_t类型的数据
     * 返回值：无
     */
    static void ParseJson(uint8_t &elem, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于uint16_t类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点uint16_t类型的数据
     * 返回值：无
     */
    static void ParseJson(uint16_t &elem, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于float类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点float类型的数据
     * 返回值：无
     */
    static void ParseJson(float &elem, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于double类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点double类型的数据
     * 返回值：无
     */
    static void ParseJson(double &elem, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于:MarkerAction类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点MarkerAction类型的数据
     * 返回值：无
     */
    static void ParseJson(mdc::visual::MarkerAction &action, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于MarkerType类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点MarkerType类型的数据
     * 返回值：无
     */
    static void ParseJson(mdc::visual::MarkerType &markerType, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于color类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点color类型的数据
     * 返回值：无
     */
    static void ParseJson(mdc::visual::ColorRGBA &color, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于Times类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点Times类型的数据
     * 返回值：无
     */
    static void ParseJson(mdc::visual::Times &times, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于Vector<ColorRGBA>类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点Vector<ColorRGBA>类型的数据
     * 返回值：无
     */
    static void ParseJson(ara::core::Vector<mdc::visual::ColorRGBA> &colors, const Json::Value &currentNode,
        const std::string &key);

    /*
     * 功能：解析Json文件中属于Header类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点Header类型的数据
     * 返回值：无
     */
    static void ParseJson(mdc::visual::Header &header, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于Point类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点Point类型的数据
     * 返回值：无
     */
    static void ParseJson(mdc::visual::Point &point, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于Vector<Point>类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点Vector<Point>类型的数据
     * 返回值：无
     */
    static void ParseJson(ara::core::Vector<mdc::visual::Point> &points, const Json::Value &currentNode,
        const std::string &key);

    /*
     * 功能：解析Json文件中属于Quaternion类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点Quaternion类型的数据
     * 返回值：无
     */
    static void ParseJson(mdc::visual::Quaternion &quaternion, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于Pose类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点Pose类型的数据
     * 返回值：无
     */
    static void ParseJson(mdc::visual::Pose &pose, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于Vector3类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点Vector3类型的数据
     * 返回值：无
     */
    static void ParseJson(mdc::visual::Vector3 &vec, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于Vector3类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点Marker类型的数据
     * 返回值：无
     */
    static void ParseJson(mdc::visual::Marker &marker, const Json::Value &currentNode);

    /*
     * 功能：解析Json文件中属于RadarState类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点RadarState类型的数据
     * 返回值：无
     */
    static void ParseJson(mdc::visual::RadarState &state, const Json::Value &currentNode, const std::string &key);

    /*
     * 功能：解析Json文件中属于Vector<RadarDetect>类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点Vector<RadarDetect>类型的数据
     * 返回值：无
     */
    static void ParseJson(ara::core::Vector<mdc::visual::RadarDetect> &detects, const Json::Value &currentNode,
        const std::string &key);

    /*
     * 功能：解析Json文件中属于RadarDetect类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点RadarDetect类型的数据
     * 返回值：无
     */
    static void ParseJson(mdc::visual::RadarDetect &detect, const Json::Value &currentNode);

    /*
     * 功能：解析Json文件中属于Vector<RadarTrack>类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点Vector<RadarTrack>类型的数据
     * 返回值：无
     */
    static void ParseJson(ara::core::Vector<mdc::visual::RadarTrack> &tracks, const Json::Value &currentNode,
        const std::string &key);

    /*
     * 功能：解析Json文件中属于RadarTrack类型的数据
     * 输入参数：Json文件中的一个节点以及节点对应的键值key
     * 输出参数：当前Json节点RadarTrack类型的数据
     * 返回值：无
     */
    static void ParseJson(mdc::visual::RadarTrack &track, const Json::Value &currentNode);

    /*
     * 功能：将地图数据存储为MarkerArray数据
     * 输入参数：JsonValue
     * 输出参数：存储了地图数据的MarkerArray
     * 返回值：void
     */
    static void ParseJson(mdc::visual::MarkerArray &markerArray, const Json::Value &currentNode);

    /*
     * 功能：将json数据存储为RadarDetectArray数据
     * 输入参数：JsonValue
     * 输出参数：存储了数据的RadarDetectArray
     * 返回值：void
     */
    static void ParseJson(mdc::visual::RadarDetectArray &detectArray, const Json::Value &currentNode);

    /*
     * 功能：将json数据存储为RadarTrackArray数据
     * 输入参数：JsonValue
     * 输出参数：存储了数据的RadarTrackArray
     * 返回值：void
     */
    static void ParseJson(mdc::visual::RadarTrackArray &trackArray, const Json::Value &currentNode);
};
#endif /* VIZ_JSON_PARSER_H */
