/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
 * Description:MViz data文件读取相关帮助类
 * Create: 2021-08-12
 */
#include "file_utils.h"

ara::core::Vector<uint8_t> FileUtils::ReadBinaryDataFile(const std::string &filePath)
{
    ara::core::Vector<uint8_t> fileContent {};
    if (ReadFile(filePath, fileContent) != ERROR && fileContent.empty()) {
        ara::log::Logger &logger = CreateLogger("VIZS", "FileHelper", ara::log::LogLevel::kVerbose);
        logger.LogWarn() << "The file is empty. The file is " << filePath;
        return {};
    }
    return fileContent;
}

Json::Value FileUtils::ReadJsonFile(const std::string &filePath)
{
    std::string fileContent {};
    ara::log::Logger &logger = CreateLogger("VIZS", "FileHelper", ara::log::LogLevel::kVerbose);
    if (ReadFile(filePath, fileContent) != ERROR && fileContent.empty()) {
        logger.LogWarn() << "The file is empty. The file is " << filePath;
        return {};
    }

    Json::CharReaderBuilder builder {};
    std::unique_ptr<Json::CharReader> reader(builder.newCharReader());
    int rawJsonLength = static_cast<int>(fileContent.length());
    Json::Value root {};
    std::string err {};
    if (!reader->parse(fileContent.c_str(), fileContent.c_str() + rawJsonLength, &root, &err)) {
        logger.LogError() << "Failed to parse the json file. The file is " << filePath << err;
        return {};
    }
    return root;
}

YAML::Node FileUtils::ReadYamlFile(const std::string &fileName)
{
    ara::log::Logger &logger = CreateLogger("VIZS", "FileHelper", ara::log::LogLevel::kVerbose);
    const std::string filePath = SystemUtils::GetFilePathInDataDir(fileName);
    YAML::Node root;

    try {
        root = YAML::LoadFile(filePath);
    } catch (YAML::BadFile &e) {
        logger.LogError() << "Failed to read the YAML file! The file path is " << filePath;
        logger.LogInfo() << "Run the \"scp\" command to copy the data folder "
                         << "in the viz_sample directory to " <<
            SystemUtils::RelativePathToAbsolutePath(SystemUtils::GetExecutablePath() + "/../");
        return YAML::Node();
    }

    return root;
}
