/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
 * Description:MViz 系统相关帮助类
 * Create: 2021-08-12
 */
#include "system_utils.h"

#include <vector>
#include <climits>

#include <unistd.h>

using namespace std;
using namespace ara::log;

mdc::visual::Times SystemUtils::GetCurrentTime()
{
    const auto now = std::chrono::high_resolution_clock::now().time_since_epoch();
    uint32_t sec = std::chrono::duration_cast<std::chrono::seconds>(now).count();
    uint32_t nsec = std::chrono::duration_cast<std::chrono::nanoseconds>(now).count() % 1000000000UL;
    return mdc::visual::Times { sec, nsec };
}

string SystemUtils::GetExecutablePath()
{
    vector<char> result(PATH_MAX, '\0');
    auto ret = readlink("/proc/self/exe", &result[0], result.capacity() - 1);
    if (!ret) {
        ara::log::Logger &logger = CreateLogger("VIZS", "SystemHelper", ara::log::LogLevel::kVerbose);
        logger.LogError() << "Failed to obtain the program execution path.";
        return {};
    }
    return string(result.data()).substr(0, string(result.data()).find_last_of('/'));
}

std::string SystemUtils::RelativePathToAbsolutePath(const string &relativePath)
{
    char absolutePath[PATH_MAX] {};
    auto ret = realpath(relativePath.data(), absolutePath);
    if (ret == nullptr) {
        ara::log::Logger &logger = CreateLogger("VIZS", "SystemHelper", ara::log::LogLevel::kVerbose);
        logger.LogError() << "Failed to obtain the absolute path.";
        return {};
    }
    return { absolutePath };
}

std::string SystemUtils::GetFilePathInDataDir(const string &fileName)
{
    return SystemUtils::GetExecutablePath() + "/../data/" + fileName;
}
