#pragma once

#include <json-glib/json-glib.h>
#include <thread>
#include <boost/asio.hpp>

#include "WebRTCPipeline.h"
#include "WebsocketClient.h"
#include "WebsocketServer.h"

class WebRTCMain : public WebsocketClientListener, WebRTCPipelineListener, WebsocketServerListener {
private:
  WebsocketClient *mClient[STREAMS];
  WebRTCPipeline *mPipeline[STREAMS];
  WebsocketServer *mServer[STREAMS];

  std::thread *_serverThreads[STREAMS];
  std::thread *_pipelineThreads[STREAMS];

 void startPipeline(gushort index);
  void stopPipeline(gushort index);
  void praseSdpAndIce(std::string& message, gushort index);
  void parseSdp(JsonObject *data_json_object, gushort index);
  void parseIce(JsonObject *data_json_object, gushort index);

public:
  WebRTCMain();
  virtual ~WebRTCMain();

  void connectSignallingServer(std::string url, std::string& origin, gushort id);
  void disconnectSignallingServer(gushort id);

  void startSignallingServer();
  void stopSignallingServer();

  // WebsocketClientListener implements.
  virtual void onConnected(WebsocketClient *client);
  virtual void onDisconnected(WebsocketClient *client);
  virtual void onMessage(WebsocketClient *client, std::string& message);

  // WebRTCPipelineListener implements.
  virtual void onSendSdp(WebRTCPipeline *pipeline, gint type, gchar *sdp_string,gushort index);
  virtual void onSendIceCandidate(WebRTCPipeline *pipeline, guint mlineindex, gchar *candidate,gushort index);
  virtual void onAddStream(WebRTCPipeline *pipeline, GstPad *pad);
  virtual void onDataChannelConnected(WebRTCPipeline *pipeline);
  virtual void onDataChannelDisconnected(WebRTCPipeline *pipeline);
  virtual void onDataChannel(WebRTCPipeline *pipeline, std::string& message);

  //WebsocketServerListener implements
  virtual void onClientConnected(WebsocketServer *server, SoupWebsocketConnection *client,gushort index);
  virtual void onClientDisconnected(WebsocketServer *server, SoupWebsocketConnection *client,gushort index);
  virtual void onClientMessage(WebsocketServer *server, SoupWebsocketConnection *client, std::string& message,gushort index);
};
