#pragma once

#include <string>
#include <vector>
#include <gst/gst.h>
#include <json-glib/json-glib.h>
#include <boost/asio.hpp>

#include "WebRTCDataChannel.h"
#include "WebRTCVideo.h"

#define UDP_HOST_IP "192.168.1.5"
#define UDP_PORT 5000

class WebRTCPipeline;

class WebRTCPipelineListener {
public:
  virtual void onSendSdp(WebRTCPipeline *pipeline, gint type, gchar *sdp_string,gushort index) {}
  virtual void onSendIceCandidate(WebRTCPipeline *pipeline, guint mlineindex, gchar *candidate,gushort index) {}
  virtual void onAddStream(WebRTCPipeline *pipeline, GstPad *pad) {}

  virtual void onDataChannelConnected(WebRTCPipeline *pipeline) {}
  virtual void onDataChannelDisconnected(WebRTCPipeline *pipeline) {}
  virtual void onDataChannel(WebRTCPipeline *pipeline, std::string& message) {}
};

struct PipeLine {
  GstElement *pipeline;
  GstElement *appsrc;
  GstElement *queue;
  GstElement *openh264enc;
  GstElement *videoconvert;
  GstElement *udpsink;
  GstElement *capsfilter;
  GstElement *rtph264depay;
  GstElement *h264parse;
  GstElement *rtph264pay;
  GstElement *h264filter;
  GstElement *timestamper;
  GstElement *webrtcbin;
  GstElement *videorate;
  GstCaps *video_caps;
  GstCaps *src_caps;
  GstBus *bus;
};

class WebRTCPipeline : public WebRTCDataChannelListener {
private:
  // GstElement *mPipeline;
  // GstElement *mWebRTCBin;
  gushort _index;
  WebRTCPipelineListener *mListener;
  WebRTCDataChannel *mSendDataChannel;
  WebRTCVideo *videoChannel;
  struct PipeLine pipeline;
  std::vector<WebRTCDataChannel*> mReceiveDataChannels;
  
  gint mNegotiationNeededHandleId;
  gint mSendIceCandidateHandleId;
  gint mIceGatheringStateNotifyHandleId;
  gint mIncomingStreamHandleId;
  gint mDataChannelHandleId;

  GMainLoop *_loop;

  void createReceiveDataChannel(GstWebRTCDataChannel *dataChannel);
  void sendSdp(GstWebRTCSessionDescription *desc);
  void sendIceCandidate(guint mlineindex, gchar *candidate);
  void addStream(GstPad *pad);
  void onOfferReceived(GstSDPMessage *sdp);
  void onAnswerReceived(GstSDPMessage *sdp);

  static void onNegotiationNeeded(GstElement *webrtcbin, gpointer userData);
  static void onSendIceCandidate(GstElement *webrtcbin, guint mlineindex, gchar *candidate, gpointer userData);
  static void onIceGatheringStateNotify(GstElement *webrtcbin, GParamSpec *pspec, gpointer userData);
  static void onIncomingStream(GstElement *webrtcbin, GstPad *pad, gpointer userData);
  static void onDataChannel(GstElement *webrtcbin, GObject *dataChannel, gpointer userData);
  static void onOfferCreated(GstPromise *promise, gpointer userData);
  static void onAnswerCreated(GstPromise *promise, gpointer userData);

public:
  WebRTCPipeline(gushort index);
  virtual ~WebRTCPipeline();

  inline void setListener(WebRTCPipelineListener *listener) {
    mListener = listener;
  }

  void startPipeline();
  void stopPipeline();
  void sendMessage(std::string& message);
  gushort getIndex();

  void onOfferReceived(const gchar *sdp);
  void onAnswerReceived(const gchar *sdp);
  void onIceReceived(guint mlineIndex, const gchar *candidateString);

  // WebRTCDataChannelListener implements.
  virtual void onConnected(WebRTCDataChannel *channel);
  virtual void onDisconnected(WebRTCDataChannel *channel);
  virtual void onMessage(WebRTCDataChannel *channel, std::string& message);
};
