#pragma once
#include <gst/gst.h>
#include <gst/app/gstappsrc.h>
#include <boost/asio.hpp>
#include <boost/bind/bind.hpp>
#include <boost/thread/thread.hpp>
#include "DataIPC.h"
#include <string>
#include <vector>
#include <mutex>

class WebRTCVideo {
    private:
        GstAppSrc *_appsrc;
        DataIPC *ipc;
        gint _apsrcNeedDataHandlerId;
        gint _apsrcEnoughDataHandlerId;
        gint _apsrcSeekDataHandlerId;
        uint8_t _index;
        std::mutex _sendMutex;
        bool _pipelineReady;
        GstClockTime timestamp;

        boost::asio::io_service _ioService;
        boost::asio::executor_work_guard<boost::asio::io_context::executor_type> _worker;
        boost::thread _thread;


        void readFiles();
        void sendFrame(const uint8_t *buffer, uint32_t bufferSize);
        void pushFrame(GstBuffer *buffer);
        bool isRegularFile(const std::string &path);
        void prepareSend(uint8_t streamIndex);
        void eventLoop();
        void slotData(std::vector<uint8_t>& data,const uint8_t streamIndex);
        void dataReady(GstBuffer* buffer,const uint8_t streamIndex);
    
    public:
        static std::string directoryPath;
        std::vector<std::string> _filenames;
        static bool fromFile;


        WebRTCVideo(GstElement *appsrc, uint8_t index);
        ~WebRTCVideo();
        void start();
        void stop();

        void readFrames(const std::string &filename);

        static void onNeedData(GstAppSrc *appsrc,guint size, gpointer userData);
        static void onEnoughData(GstAppSrc *src, gpointer userData);
        static gboolean onSeekData(GstAppSrc *src, guint64 offset, gpointer userData);
};