#include "DataIPC.h"
#include <iostream>
#include <cstring>

Subscriber::Subscriber(): proxySocA_camera{nullptr}
{
    ara::log::InitLogging("DEAC", "CM_DDS_EVENT_ASYNC_CLIENT_SAMPLE", ara::log::LogLevel::kVerbose,
        (ara::log::LogMode::kConsole | ara::log::LogMode::kRemote));
    handler = ProxySocA::StartFindService(
        [this](ara::com::ServiceHandleContainer<ProxySocA::HandleType> handles, ara::com::FindServiceHandle handler)
        {
            Subscriber::availabilityCallback(std::move(handles), handler);
        },
        ara::core::InstanceSpecifier(
            "visualization/visualization/CameraH265ImageRPort"));
}

Subscriber::~Subscriber()
{
    stop();
}

void Subscriber::availabilityCallback(ara::com::ServiceHandleContainer<ProxySocA::HandleType> handles,
												 ara::com::FindServiceHandle handler) {

    log_.LogInfo() << "SOC_A cam serviceAvailabilityCallback.";
    for (auto it : handles)
    {
        std::lock_guard<std::mutex> lock(proxySocAMutex_camera);
        if (it.GetInstanceId().ToString() == "1" && proxySocA_camera == nullptr)
        {
            log_.LogInfo() << "Created camProxySocA_ instance ";
            proxySocA_camera = std::make_shared<ProxySocA>(it);
            proxySocA_camera->CameraH265ImagePDataViz.SetReceiveHandler([this]() { Subscriber::imageCallback(); });
            const size_t maxSampleCount = 10U;
            proxySocA_camera->CameraH265ImagePDataViz.Subscribe(maxSampleCount);
        }
    }
}

void Subscriber::imageCallback()
{
    // log_.LogInfo() << "CameraImage event on soc_a received.";
    // std::cout <<"imageCallback: tid ="<< std::this_thread::get_id() << std::endl;
    proxySocA_camera->CameraH265ImagePDataViz.GetNewSamples([this](ara::com::SamplePtr<CameraH265Image const> ptr)
                                                {
    auto sample = ptr.Get();
    if (sample->cameraId < STREAMS){
        GstBuffer *buffer = gst_buffer_new_allocate(NULL, sample->rawData.size(), NULL);
        gst_buffer_fill(buffer, 0, sample->rawData.data(), sample->rawData.size());
        bufferReady(buffer, sample->cameraId);
        // _data[sample->cameraId].assign(sample->rawData.begin(), sample->rawData.end());
        // _dataReady(_data[sample->cameraId], sample->cameraId);
    }
    // log_.LogInfo() << "camera id: " << sample->cameraId; 
    });
}

void Subscriber::stop() {
    if (proxySocA_camera != nullptr && proxySocA_camera->CameraH265ImagePDataViz.IsSubscribed())
    {
        proxySocA_camera->CameraH265ImagePDataViz.UnsetReceiveHandler();
        proxySocA_camera->CameraH265ImagePDataViz.Unsubscribe();
    }
    ProxySocA::StopFindService(handler);
}

// const std::vector<uint8_t>& Subscriber::GetData(uint8_t streamIndex) {
//     if (streamIndex >= STREAMS)
//         return _data[0];
//     // std::lock_guard<std::mutex> lock(dataMutex);
//     return _data[streamIndex];
// }
