#include "WebRTCPipeline.h"
#include <gst/gst.h>
#include <gst/sdp/sdp.h>
#define GST_USE_UNSTABLE_API
#include <gst/webrtc/webrtc.h>
#include <iostream>

static gboolean onPipelineMessage (GstBus * bus, GstMessage * message, void * app) {
  switch (GST_MESSAGE_TYPE (message)) {
    case GST_MESSAGE_EOS:
      g_print ("Received End of Stream message\n");
      break;
    case GST_MESSAGE_ERROR:
      {
        g_print ("Received error\n");

        GError *err = NULL;
        gchar *dbg_info = NULL;

        gst_message_parse_error (message, &err, &dbg_info);
        g_printerr ("ERROR from element %s: %s\n",
            GST_OBJECT_NAME (message->src), err->message);
        g_printerr ("Debugging info: %s\n", (dbg_info) ? dbg_info : "none");
        g_error_free (err);
        g_free (dbg_info);
      }

      break;
    case GST_MESSAGE_STATE_CHANGED:
          // gst_element_get_state(app->source, &state, &pending, GST_CLOCK_TIME_NONE);
      /* g_print ("State changed from %i to %i\n", state, pending); */
      break;
        default:
      break;
  }
  return TRUE;
}

WebRTCPipeline::WebRTCPipeline(gushort index)
{
  mListener = nullptr;
  mSendDataChannel = nullptr;
  _index = index;
  mNegotiationNeededHandleId = 0;
  mSendIceCandidateHandleId = 0;
  mIceGatheringStateNotifyHandleId = 0;
  mIncomingStreamHandleId = 0;
  mDataChannelHandleId = 0;
}

WebRTCPipeline::~WebRTCPipeline()
{
  stopPipeline();
}

void WebRTCPipeline::startPipeline()
{
  pipeline.pipeline = gst_pipeline_new("mypipeline");
  pipeline.video_caps = gst_caps_new_simple("application/x-rtp",
                                "media", G_TYPE_STRING, "video",
                                "clock-rate", G_TYPE_UINT, 90000,
                                "encoding-name", G_TYPE_STRING, "H264",
                                "payload", G_TYPE_UINT, 96, NULL);
  pipeline.src_caps = gst_caps_new_simple("video/x-h264",
                                "stream-format", G_TYPE_STRING, "byte-stream",
                                "framerate", GST_TYPE_FRACTION, 15, 1,
                                NULL);
  pipeline.appsrc = gst_element_factory_make("appsrc", NULL);
  g_object_set(pipeline.appsrc, "is-live", TRUE, NULL);
  g_object_set(pipeline.appsrc, "block", TRUE, NULL);
  g_object_set(pipeline.appsrc, "emit-signals", FALSE, NULL);
  g_object_set(pipeline.appsrc, "max-latency", 100, NULL);
  g_object_set(pipeline.appsrc, "min-latency", 0, NULL);
  g_object_set(pipeline.appsrc, "format", GST_FORMAT_TIME, NULL);
  g_object_set(pipeline.appsrc, "max-bytes", 10000, NULL);
  g_object_set(pipeline.appsrc, "max-buffers", 10, NULL);
  pipeline.queue = gst_element_factory_make("queue", NULL);
  pipeline.capsfilter = gst_element_factory_make("capsfilter", NULL);
  g_object_set(pipeline.capsfilter, "caps", pipeline.video_caps, NULL);
  pipeline.h264parse = gst_element_factory_make("h264parse", NULL);
  pipeline.rtph264pay = gst_element_factory_make("rtph264pay", NULL);
  g_object_set(pipeline.rtph264pay, "pt", 96, "aggregate-mode", 1, NULL);//1= zero latency
  pipeline.timestamper = gst_element_factory_make("h264timestamper", NULL);
  pipeline.h264filter = gst_element_factory_make("capsfilter", NULL);
  g_object_set(pipeline.h264filter, "caps", pipeline.src_caps, NULL);
  pipeline.videorate = gst_element_factory_make("videorate", NULL);
  g_object_set(pipeline.videorate, "max-rate", 15, NULL);
  pipeline.webrtcbin = gst_element_factory_make("webrtcbin", NULL);
  g_object_set(pipeline.webrtcbin, "bundle-policy", GST_WEBRTC_BUNDLE_POLICY_MAX_BUNDLE,
              "latency", 100, "turn-server", "turn://swm:123@112.27.121.65:3478", NULL);
  pipeline.udpsink = gst_element_factory_make("udpsink", NULL);
  g_object_set(pipeline.udpsink, "host", UDP_HOST_IP, "port", UDP_PORT, NULL);

  pipeline.rtph264depay = gst_element_factory_make("rtph264depay", NULL);
  gst_bin_add_many(GST_BIN(pipeline.pipeline),
                  pipeline.appsrc, 
                  // pipeline.queue, 
                  // pipeline.h264filter, 
                  pipeline.h264parse, 
                  // pipeline.timestamper, 
                  pipeline.rtph264pay, 
                  pipeline.videorate,
                  // pipeline.capsfilter,
                  pipeline.webrtcbin, NULL);
  gst_element_link_many(pipeline.appsrc,
                      // pipeline.queue,
                      pipeline.h264parse, 
                      pipeline.videorate,
                      // pipeline.h264filter,
                      pipeline.rtph264pay, 
                      pipeline.webrtcbin, NULL);
  std::string debugfile = std::string("pipeline") + std::to_string(_index);
  gst_debug_bin_to_dot_file (GST_BIN(pipeline.pipeline),
                           GST_DEBUG_GRAPH_SHOW_NON_DEFAULT_PARAMS,
                           debugfile.c_str());
  
  pipeline.bus = gst_element_get_bus (pipeline.pipeline);
  gst_bus_add_watch (pipeline.bus, (GstBusFunc) onPipelineMessage, NULL);
  gst_object_unref (pipeline.bus);

  // 接続するためのネゴシエーションを行うためのコールバックを設定
  mNegotiationNeededHandleId = g_signal_connect(pipeline.webrtcbin, "on-negotiation-needed", 
      G_CALLBACK(WebRTCPipeline::onNegotiationNeeded), this);
  mSendIceCandidateHandleId = g_signal_connect(pipeline.webrtcbin, "on-ice-candidate", 
      G_CALLBACK(WebRTCPipeline::onSendIceCandidate), this);
  mIceGatheringStateNotifyHandleId = g_signal_connect(pipeline.webrtcbin, "notify::ice-gathering-state", 
      G_CALLBACK(WebRTCPipeline::onIceGatheringStateNotify), this);


  // 映像受信用のコールバック
  mIncomingStreamHandleId = g_signal_connect(pipeline.webrtcbin, "pad-added", 
      G_CALLBACK(WebRTCPipeline::onIncomingStream), this);

  // // 受信用データチャンネルのコールバック
  mDataChannelHandleId = g_signal_connect(pipeline.webrtcbin, "on-data-channel", 
      G_CALLBACK(WebRTCPipeline::onDataChannel), this);

  // 送信用データチャンネルを作成
  std::string name("send-channel");
  mSendDataChannel = new WebRTCDataChannel(pipeline.webrtcbin);
  mSendDataChannel->setListener(this);
  mSendDataChannel->connect(name);

  // appsrc = gst_bin_get_by_name(GST_BIN(mPipeline), "app");

  videoChannel = new WebRTCVideo(pipeline.appsrc, _index);
  videoChannel->start();


  // パイプラインの再生を開始
  gst_element_set_state(GST_ELEMENT(pipeline.pipeline), GST_STATE_PLAYING);
  _loop = g_main_loop_new(NULL, FALSE);
  g_main_loop_run(_loop);
}

void WebRTCPipeline::stopPipeline()
{
  if(pipeline.pipeline) {
    gst_element_set_state(GST_ELEMENT(pipeline.pipeline), GST_STATE_NULL);
  }
  if (mNegotiationNeededHandleId) {
    g_signal_handler_disconnect(G_OBJECT(pipeline.webrtcbin), mNegotiationNeededHandleId);
    mNegotiationNeededHandleId = 0;
  }

  if (mSendIceCandidateHandleId) {
    g_signal_handler_disconnect(G_OBJECT(pipeline.webrtcbin), mSendIceCandidateHandleId);
    mSendIceCandidateHandleId = 0;
  }

  if (mIceGatheringStateNotifyHandleId) {
    g_signal_handler_disconnect(G_OBJECT(pipeline.webrtcbin), mIceGatheringStateNotifyHandleId);
    mIceGatheringStateNotifyHandleId = 0;
  }

  if (mIncomingStreamHandleId) {
    g_signal_handler_disconnect(G_OBJECT(pipeline.webrtcbin), mIncomingStreamHandleId);
    mIncomingStreamHandleId = 0;
  }

  if (mDataChannelHandleId) {
    g_signal_handler_disconnect(G_OBJECT(pipeline.webrtcbin), mDataChannelHandleId);
    mDataChannelHandleId = 0;
  }

  if (mSendDataChannel) {
    delete mSendDataChannel;
    mSendDataChannel = nullptr;
  }

  for (auto itr = mReceiveDataChannels.begin(); itr != mReceiveDataChannels.end(); ++itr) {
    delete *itr;
  }
  mReceiveDataChannels.clear();

  if (pipeline.webrtcbin) {
    gst_object_unref(pipeline.webrtcbin);
    pipeline.webrtcbin = nullptr;
  }

  if (pipeline.pipeline) {
    gst_element_set_state(GST_ELEMENT(pipeline.pipeline), GST_STATE_NULL);
    g_clear_object(&pipeline.pipeline);
    pipeline.pipeline = nullptr;
  }


  if (pipeline.appsrc) {
    gst_element_set_state(GST_ELEMENT(pipeline.appsrc), GST_STATE_NULL);
    g_clear_object(&pipeline.appsrc);
    pipeline.appsrc = nullptr;
  }


  if (videoChannel) {
    // videoChannel->stop();
    delete videoChannel;
    videoChannel = nullptr;
  }
  
  if(_loop) {
    g_main_loop_quit(_loop);
    g_main_loop_unref(_loop);
    _loop = nullptr;
  }

}

void WebRTCPipeline::sendMessage(std::string& message)
{
  if (mSendDataChannel) {
    mSendDataChannel->sendMessage(message);
  }
}

void WebRTCPipeline::onOfferReceived(const gchar *sdpString) 
{
  GstSDPMessage *sdp = NULL;

  int ret = gst_sdp_message_new(&sdp);
  g_assert_cmphex(ret, ==, GST_SDP_OK);

  ret = gst_sdp_message_parse_buffer((guint8 *) sdpString, strlen(sdpString), sdp);
  if (ret != GST_SDP_OK) {
    g_error ("Could not parse SDP string\n");
    return;
  }

  onOfferReceived(sdp);
}

void WebRTCPipeline::onAnswerReceived(const gchar *sdpString)
{
  GstSDPMessage *sdp = NULL;

  int ret = gst_sdp_message_new(&sdp);
  g_assert_cmphex(ret, ==, GST_SDP_OK);

  ret = gst_sdp_message_parse_buffer((guint8 *) sdpString, strlen(sdpString), sdp);
  if (ret != GST_SDP_OK) {
    g_error ("Could not parse SDP string\n");
    return;
  }

  onAnswerReceived(sdp);
}

void WebRTCPipeline::onIceReceived(guint mlineIndex, const gchar *candidateString)
{
  g_signal_emit_by_name(pipeline.webrtcbin, "add-ice-candidate", mlineIndex, candidateString);
}

// private functions.

void WebRTCPipeline::onAnswerReceived(GstSDPMessage *sdp)
{
  GstWebRTCSessionDescription *answer = gst_webrtc_session_description_new(GST_WEBRTC_SDP_TYPE_ANSWER, sdp);
  if (answer) {
    GstPromise *promise = gst_promise_new();
    g_signal_emit_by_name(pipeline.webrtcbin, "set-remote-description", answer, promise);
    gst_promise_interrupt(promise);
    gst_promise_unref(promise);
    gst_webrtc_session_description_free(answer);
  }
}

void WebRTCPipeline::onOfferReceived(GstSDPMessage *sdp)
{
  GstWebRTCSessionDescription *offer = gst_webrtc_session_description_new(GST_WEBRTC_SDP_TYPE_OFFER, sdp);
  if (offer) {
    GstPromise *promise = gst_promise_new();
    g_signal_emit_by_name(pipeline.webrtcbin, "set-remote-description", offer, promise);
    gst_promise_interrupt(promise);
    gst_promise_unref(promise);
    gst_webrtc_session_description_free(offer);

    promise = gst_promise_new_with_change_func(WebRTCPipeline::onAnswerCreated, this, NULL);
    g_signal_emit_by_name(pipeline.webrtcbin, "create-answer", NULL, promise);
  }
}

void WebRTCPipeline::sendSdp(GstWebRTCSessionDescription *desc)
{
  if (mListener) {
    gchar *sdp_string = gst_sdp_message_as_text(desc->sdp);
    mListener->onSendSdp(this, desc->type, sdp_string, _index);
    g_free(sdp_string);
  }
}

void WebRTCPipeline::sendIceCandidate(guint mlineindex, gchar *candidate)
{
  if (mListener) {
    mListener->onSendIceCandidate(this, mlineindex, candidate, _index);
  }
}

void WebRTCPipeline::addStream(GstPad *pad)
{
  if (mListener) {
    mListener->onAddStream(this, pad);
  }
}

void WebRTCPipeline::createReceiveDataChannel(GstWebRTCDataChannel *dataChannel)
{
  WebRTCDataChannel *channel = new WebRTCDataChannel(pipeline.webrtcbin);
  if (channel) {
    channel->setListener(this);
    channel->connect(dataChannel);
    mReceiveDataChannels.push_back(channel);
  }
}

// callback static functions.

void WebRTCPipeline::onNegotiationNeeded(GstElement *webrtcbin, gpointer userData)
{
  WebRTCPipeline *pipeline = (WebRTCPipeline *) userData;
  GstPromise *promise = gst_promise_new_with_change_func(WebRTCPipeline::onOfferCreated, userData, NULL);
  g_signal_emit_by_name(pipeline->pipeline.webrtcbin, "create-offer", NULL, promise);
}

// offer が作成された場合
void WebRTCPipeline::onOfferCreated(GstPromise *promise, gpointer userData)
{
  WebRTCPipeline *pipeline = (WebRTCPipeline *) userData;

  g_assert_cmphex(gst_promise_wait(promise), ==, GST_PROMISE_RESULT_REPLIED);

  GstWebRTCSessionDescription *offer = NULL;

  const GstStructure *reply = gst_promise_get_reply(promise);
  gst_structure_get(reply, "offer", GST_TYPE_WEBRTC_SESSION_DESCRIPTION, &offer, NULL);
  gst_promise_unref(promise);

  promise = gst_promise_new();
  g_signal_emit_by_name(pipeline->pipeline.webrtcbin, "set-local-description", offer, promise);
  gst_promise_interrupt(promise);
  gst_promise_unref(promise);

  pipeline->sendSdp(offer);

  gst_webrtc_session_description_free(offer);
}

void WebRTCPipeline::onAnswerCreated(GstPromise *promise, gpointer userData)
{
  WebRTCPipeline *pipeline = (WebRTCPipeline *) userData;

  g_assert_cmphex(gst_promise_wait(promise), ==, GST_PROMISE_RESULT_REPLIED);

  GstWebRTCSessionDescription *answer = NULL;

  const GstStructure *reply = gst_promise_get_reply(promise);
  gst_structure_get(reply, "answer", GST_TYPE_WEBRTC_SESSION_DESCRIPTION, &answer, NULL);
  gst_promise_unref(promise);

  promise = gst_promise_new();
  g_signal_emit_by_name(pipeline->pipeline.webrtcbin, "set-local-description", answer, promise);
  gst_promise_interrupt(promise);
  gst_promise_unref(promise);

  pipeline->sendSdp(answer);

  gst_webrtc_session_description_free(answer);
}

void WebRTCPipeline::onSendIceCandidate(GstElement *webrtcbin, guint mlineindex, gchar *candidate, gpointer userData)
{
  WebRTCPipeline *pipeline = (WebRTCPipeline *) userData;
  pipeline->sendIceCandidate(mlineindex, candidate);
}

void WebRTCPipeline::onIceGatheringStateNotify(GstElement *webrtcbin, GParamSpec *pspec, gpointer userData)
{
  GstWebRTCICEGatheringState ice_gather_state;
  const gchar *new_state = "unknown";

  g_object_get(webrtcbin, "ice-gathering-state", &ice_gather_state, NULL);
  switch (ice_gather_state) {
  case GST_WEBRTC_ICE_GATHERING_STATE_NEW:
    new_state = "new";
    break;
  case GST_WEBRTC_ICE_GATHERING_STATE_GATHERING:
    new_state = "gathering";
    break;
  case GST_WEBRTC_ICE_GATHERING_STATE_COMPLETE:
    new_state = "complete";
    break;
  }
  g_print("ICE gathering state changed to %s.\n", new_state);
}

// 新規ストリームの追加
void WebRTCPipeline::onIncomingStream(GstElement *webrtcbin, GstPad *pad, gpointer userData)
{
  WebRTCPipeline *pipeline = (WebRTCPipeline *) userData;
  pipeline->addStream(pad);
}

// 新規データチャンネルの接続
void WebRTCPipeline::onDataChannel(GstElement *webrtcbin, GObject *dataChannel, gpointer userData)
{
  WebRTCPipeline *pipeline = (WebRTCPipeline *) userData;
  pipeline->createReceiveDataChannel((GstWebRTCDataChannel *)dataChannel);
}

// WebRTCDataChannelListener implements.

void WebRTCPipeline::onConnected(WebRTCDataChannel *channel)
{
  // MEMO データチャンネルが接続されたときに呼びだれます。

  if (mListener) {
    mListener->onDataChannelConnected(this);
  }
}

void WebRTCPipeline::onDisconnected(WebRTCDataChannel *channel)
{
  // MEMO データチャンネルが切断されたときに呼びだれます。
  
  if (mListener) {
    mListener->onDataChannelDisconnected(this);
  }
}

void WebRTCPipeline::onMessage(WebRTCDataChannel *channel, std::string& message)
{
  // MEMO データチャンネルにメッセージ送られてきた時に呼び出されます。

  if (mListener) {
    mListener->onDataChannel(this, message);
  }
}