#include "WebRTCVideo.h"

#include <glob.h>
#include <sys/stat.h> 
#include <algorithm>
#include <fstream>
#include <iostream>
#include <thread>


std::string WebRTCVideo::directoryPath;
bool WebRTCVideo::fromFile;

WebRTCVideo::WebRTCVideo(GstElement *appsrc, uint8_t index):
 _appsrc{GST_APP_SRC(appsrc)}, _index{index}, _pipelineReady{false},
 timestamp{0},_worker{boost::asio::make_work_guard(_ioService)}
{
    if(fromFile)
        readFiles();
    else {
        // ipc = new SharedMemoryIPC("camera_driving_webrtc_ipc_buf",1);
        ipc = new Subscriber();
        // ipc->DataReady().connect(boost::bind(&WebRTCVideo::slotData, this,
        //                 boost::placeholders::_1,boost::placeholders::_2));
        ipc->BufferReady().connect(boost::bind(&WebRTCVideo::dataReady, this,
                boost::placeholders::_1,boost::placeholders::_2));
    }
}

void WebRTCVideo::slotData(std::vector<uint8_t>& data,const uint8_t streamIndex) {
    if (_index == streamIndex)
        _ioService.post(boost::bind(&WebRTCVideo::sendFrame, this, data.data(), data.size()));
}

void WebRTCVideo::dataReady(GstBuffer* buffer,const uint8_t streamIndex) {
    if (_index == streamIndex)
        _ioService.post(boost::bind(&WebRTCVideo::pushFrame, this, buffer));
}

void WebRTCVideo::eventLoop() {
    _ioService.run();
}

void WebRTCVideo::start(){
    _thread = boost::thread(boost::bind(&WebRTCVideo::eventLoop, this));
    _apsrcNeedDataHandlerId = g_signal_connect(_appsrc, "need-data",
        G_CALLBACK(WebRTCVideo::onNeedData),this);
    _apsrcEnoughDataHandlerId = g_signal_connect(_appsrc, "enough-data",
        G_CALLBACK(WebRTCVideo::onEnoughData),this);
    _apsrcSeekDataHandlerId = g_signal_connect(_appsrc, "seek-data",
        G_CALLBACK(WebRTCVideo::onSeekData),this);
}

void WebRTCVideo::stop() {
    if (_apsrcNeedDataHandlerId) {
        g_signal_handler_disconnect(G_OBJECT(_appsrc), _apsrcNeedDataHandlerId);
        _apsrcNeedDataHandlerId = 0;
    }
    if (_apsrcEnoughDataHandlerId) {
        g_signal_handler_disconnect(G_OBJECT(_appsrc), _apsrcEnoughDataHandlerId);
        _apsrcEnoughDataHandlerId = 0;
    }
    if (_apsrcSeekDataHandlerId) {
        g_signal_handler_disconnect(G_OBJECT(_appsrc), _apsrcSeekDataHandlerId);
        _apsrcSeekDataHandlerId = 0;
    }
}

void WebRTCVideo::readFiles() {
    
    glob_t glob_result;
    directoryPath = directoryPath + "/*";
    glob(directoryPath.c_str(), GLOB_TILDE, nullptr, &glob_result);
    for (size_t i = 0; i < glob_result.gl_pathc; ++i) {
      if (isRegularFile(glob_result.gl_pathv[i])) {
        _filenames.push_back(glob_result.gl_pathv[i]);
    }
    }

    globfree(&glob_result);
    std::sort(_filenames.begin(), _filenames.end());
}

bool WebRTCVideo::isRegularFile(const std::string &path) {
    struct stat stat_buf;
    return stat(path.c_str(), &stat_buf) == 0 && S_ISREG(stat_buf.st_mode);
}

void WebRTCVideo::readFrames(const std::string &filename) {
    std::ifstream frame_file(filename, std::ios::binary);

    if (!frame_file.is_open()) {
        std::cerr << "file "<<filename <<" can not be open" <<std::endl;
        return;
    }

    frame_file.seekg(0, std::ios::end);
    int file_size = frame_file.tellg();

    char* data_buffer = new char[file_size];

    frame_file.seekg(0, std::ios::beg);
    frame_file.read(data_buffer, file_size);

    sendFrame((uint8_t *)data_buffer, file_size);

    delete[] data_buffer;
    frame_file.close();
}

void WebRTCVideo::sendFrame(const uint8_t *buffer, uint32_t bufferSize) {
    // if (!_pipelineReady)
    //     return;
    static GstClockTime timestamp = 0;
    // std::cout <<"slot: tid= "<< std::this_thread::get_id() << std::endl;
    GstBuffer *_buffer = gst_buffer_new_allocate(NULL, bufferSize, NULL);
    gst_buffer_fill(_buffer, 0, buffer, bufferSize);
    GST_BUFFER_PTS (_buffer) = timestamp;
    GST_BUFFER_DURATION (_buffer) = gst_util_uint64_scale_int (1, GST_SECOND, 30);

    timestamp += GST_BUFFER_DURATION (_buffer);
    GstFlowReturn ret = gst_app_src_push_buffer(_appsrc, _buffer);
    // gst_buffer_unref(_buffer);
    switch (ret)
    {
    case GST_FLOW_OK:
        // std::cout<<"SendFrame: GST_FLOW_OK,"<<bufferSize<<std::endl;
        break;
        case GST_FLOW_FLUSHING:
        std::cout<<"SendFrame: GST_FLOW_FLUSHING,"<<bufferSize<<std::endl;
        break;
        case GST_FLOW_EOS:
        std::cout<<"SendFrame: GST_FLOW_EOS,"<<bufferSize<<std::endl;
        break;
    
    default:
        break;
    }
}

void WebRTCVideo::pushFrame(GstBuffer *buffer) {
    GST_BUFFER_PTS (buffer) = timestamp;
    GST_BUFFER_DURATION (buffer) = gst_util_uint64_scale_int (1, GST_SECOND, 30);

    timestamp += GST_BUFFER_DURATION (buffer);
    GstFlowReturn ret = gst_app_src_push_buffer(_appsrc, buffer);
    switch (ret)
    {
    case GST_FLOW_OK:
        // std::cout<<"SendFrame: GST_FLOW_OK,"<<std::endl;
        break;
        case GST_FLOW_FLUSHING:
        std::cout<<"SendFrame: GST_FLOW_FLUSHING,"<<std::endl;
        break;
        case GST_FLOW_EOS:
        std::cout<<"SendFrame: GST_FLOW_EOS,"<<std::endl;
        break;
    
    default:
        break;
    }
}

void WebRTCVideo::onNeedData(GstAppSrc *appsrc,guint size, gpointer userData){
    WebRTCVideo *_this = (WebRTCVideo *) userData;
    if(_this == nullptr)
        return;
    // _this->_sendMutex[_this->_index].lock();
    
    // g_print("In %s: tid = ", __func__);
    // std::cout << std::this_thread::get_id() << std::endl;
    _this->_pipelineReady = true;

    if (_this->fromFile){
        static guint32 index = 0;
        
        _this->readFrames(_this->_filenames.at(index % _this->_filenames.size()));
        index = (index + 1) % _this->_filenames.size();
        std::this_thread::sleep_for(std::chrono::milliseconds(33));
        return;
    } 

    // _this->sendFrame(_this->ipc->GetData(_this->_index).data(), 
    //                  _this->ipc->GetData(_this->_index).size());
    // _this->_sendMutex[_this->_index].unlock();
    // std::this_thread::sleep_for(std::chrono::milliseconds(33));
}

void WebRTCVideo::onEnoughData(GstAppSrc *src, gpointer userData) {
    // g_print("In %s: tid = ", __func__);
    // std::cout << std::this_thread::get_id() << std::endl;
    WebRTCVideo *_this = (WebRTCVideo *) userData;
    if(_this == nullptr)
        return;
    _this->_pipelineReady = false;
}

gboolean WebRTCVideo::onSeekData(GstAppSrc *src, guint64 offset, gpointer userData) {
    g_print("In %s\n", __func__);
    return TRUE;
}

WebRTCVideo::~WebRTCVideo() {
    if (ipc)
        delete ipc;
}

void WebRTCVideo::prepareSend(uint8_t streamIndex) {
    if (_index != streamIndex)
        return;
    if (!_pipelineReady)
        return;
     g_print("In %s\n", __func__);
    // _sendMutex.lock();
    // sendFrame(ipc->GetData(_index).data(), 
    //                  ipc->GetData(_index).size());
    // _sendMutex.unlock();
}