#include "WebsocketServer.h"
#include <gst/gst.h>
#include <iostream>
#include <algorithm>

WebsocketServer::WebsocketServer(WebsocketServerListener *listener, 
gushort port, gushort index):
 addr{nullptr}, port{port}, server{nullptr}, 
 _index{index}, _listener{listener} {

}

gushort WebsocketServer::getIndex() {
    return _index;
}

void WebsocketServer::onMessage(SoupWebsocketConnection *ws, SoupWebsocketDataType type, GBytes *message, gpointer userData){
    gsize len;
    WebsocketServer *_this = (WebsocketServer *)userData;
    if (!_this)
        return;
    const char *content = (const char *)g_bytes_get_data (message, &len);
    std::cout<< content <<std::endl;
    gchar *text = g_strndup(content, len);
    
    if(_this->_listener && text) {
        std::string msg = std::string(text);
        _this->_listener->onClientMessage(_this, ws, msg, _this->getIndex());
    }
    g_free(text);
}

void WebsocketServer::onIncommingConnection (SoupServer *server, SoupWebsocketConnection *connection, const char *path, SoupClientContext *client, gpointer user_data){
    WebsocketServer *websocket = (WebsocketServer *)user_data;
    if(websocket == nullptr) {
        std::cerr<<"invalid user data" <<__func__<<std::endl;
    }
    websocket->clients.push_back(g_object_ref (connection));
    std::cout<<__func__<<std::endl;
    websocket->sendMessage(connection,"HELLO");
    if (websocket->_listener)
        websocket->_listener->onClientConnected(websocket, connection, websocket->getIndex());
    g_signal_connect (connection, "message", G_CALLBACK(WebsocketServer::onMessage), user_data);
    g_signal_connect (connection, "closing", G_CALLBACK(WebsocketServer::onClosing), user_data);
    g_signal_connect (connection, "closed", G_CALLBACK(WebsocketServer::onClosed), user_data);
}

void WebsocketServer::StartServer() {
    GError *error = NULL;

    server = soup_server_new(SOUP_SERVER_SERVER_HEADER, "webrtc", NULL);
    soup_server_add_websocket_extension(server, SOUP_TYPE_WEBSOCKET_EXTENSION);
    addr = g_inet_socket_address_new_from_string ("127.0.0.1", port);
    port = g_inet_socket_address_get_port (G_INET_SOCKET_ADDRESS (addr));
    soup_server_listen_all (server, port, SOUP_SERVER_LISTEN_IPV4_ONLY, &error);
	g_assert_no_error (error);
    soup_server_add_websocket_handler (server, "/",
					   NULL, NULL, WebsocketServer::onIncommingConnection,
                        this, NULL);
    _loop = g_main_loop_new(NULL, FALSE);
    g_main_loop_run(_loop);
}

void WebsocketServer::stopServer() {
    if(server)
        soup_server_disconnect(server);
    if(_loop) {
        g_main_loop_quit(_loop);
        g_main_loop_unref(_loop);
    }
    g_clear_object(&server);
    server = nullptr;
    _loop = nullptr;
}

void WebsocketServer::sendMessage(SoupWebsocketConnection * connection,std::string message){
    if (connection)
        soup_websocket_connection_send_text(connection, message.c_str());

}

void WebsocketServer::sendMessageToAll(std::string message) {
    for (auto& con : clients) {
        soup_websocket_connection_send_text(con, message.c_str());
    }
}

void WebsocketServer::onClosing (SoupWebsocketConnection* connection, gpointer user_data) {
    WebsocketServer *_this = (WebsocketServer *)user_data;
    if (_this)
        _this->sendMessage(connection, "BYE");
}

void WebsocketServer::onClosed (SoupWebsocketConnection* connection, gpointer user_data) {
    WebsocketServer *_this = (WebsocketServer *)user_data;
    if (!_this)
        return;
    if (_this->_listener)
        _this->_listener->onClientDisconnected(_this, connection, _this->getIndex());
    auto it = std::find(_this->clients.begin(), _this->clients.end(), connection);
    if (it != _this->clients.end()) {
         _this->clients.erase(it);
    }        
}